==> GotoBLAS-1.00.tar.gz: GotoBLAS/level2/trsv/trsv_U.c <==
/*********************************************************************/
/*                                                                   */
/*             Optimized BLAS libraries                              */
/*                     By Kazushige Goto <kgoto@tacc.utexas.edu>     */
/*                                                                   */
/* Copyright (c) The University of Texas, 2005. All rights reserved. */
/* UNIVERSITY EXPRESSLY DISCLAIMS ANY AND ALL WARRANTIES CONCERNING  */
/* THIS SOFTWARE AND DOCUMENTATION, INCLUDING ANY WARRANTIES OF      */
/* MERCHANTABILITY, FITNESS FOR ANY PARTICULAR PURPOSE,              */
/* NON-INFRINGEMENT AND WARRANTIES OF PERFORMANCE, AND ANY WARRANTY  */
/* THAT MIGHT OTHERWISE ARISE FROM COURSE OF DEALING OR USAGE OF     */
/* TRADE. NO WARRANTY IS EITHER EXPRESS OR IMPLIED WITH RESPECT TO   */
/* THE USE OF THE SOFTWARE OR DOCUMENTATION.                         */
/* Under no circumstances shall University be liable for incidental, */
/* special, indirect, direct or consequential damages or loss of     */
/* profits, interruption of business, or related expenses which may  */
/* arise from use of Software or Documentation, including but not    */
/* limited to those resulting from defects in Software and/or        */
/* Documentation, or loss or inaccuracy of data of any kind.         */
/*********************************************************************/

#include <stdio.h>
#include <ctype.h>
#include "common.h"

const static FLOAT dm1 = -1.;

int CNAME(long m, FLOAT *a, long lda, FLOAT *b, long incb, void *buffer){

  long i, is, imax;
  FLOAT *aoffset1, *aoffset2, *boffset1, *boffset2;
  FLOAT *subbuffer = buffer + SUB_BUFFER_OFFSET;
  FLOAT *B;

  B = b;

  if (incb != 1) {
    B = subbuffer;
    COPY_K(m, b, incb, subbuffer, 1);
  }
  
  for (is = m; is > 0; is -= TRSV_P1){

    imax = MAX(is - TRSV_P1, 0);

    aoffset1 = a + imax * lda;
    boffset1 = B + imax;
    boffset2 = B + is;

#ifdef TRANSA
    if (m > is){
      GEMV_T(m - is, MIN(is, TRSV_P1), 0, dm1, 
	     aoffset1 + is, lda,
	     boffset2     , 1,
	     boffset1     , 1, buffer);
    }
#endif
    
    aoffset2 = a + (is - 1) + (is - 1) * lda;
    
    for (i = is - 1; i >= imax; i--) {
      
#ifndef TRANSA
      *(boffset2 - 1) -= DOT_K(is - 1 - i, aoffset2 + lda, lda,
			       boffset2, 1);
#else
      *(boffset2 - 1) -= DOT_K(is - 1 - i, aoffset2 + 1  , 1,
			       boffset2, 1);
#endif
      
#ifndef UNIT
      *(boffset2 - 1)  /= *aoffset2;
#endif
      
      boffset2 -= 1;
      aoffset2 -= 1 + lda;
    }
    
#ifndef TRANSA
    if (imax > 0){
      GEMV_N(imax, TRSV_P1, 0, dm1, aoffset1, lda,  boffset1, 1,  B, 1, buffer);
    }
#endif
    
  }

  if (incb != 1) {
    COPY_K(m, subbuffer, 1, b, incb);
  }

  return 0;
}

==> GotoBLAS-1.10.tar.gz: GotoBLAS/level2/trsv/trsv_U.c <==
/*********************************************************************/
/*                                                                   */
/*             Optimized BLAS libraries                              */
/*                     By Kazushige Goto <kgoto@tacc.utexas.edu>     */
/*                                                                   */
/* Copyright (c) The University of Texas, 2005. All rights reserved. */
/* UNIVERSITY EXPRESSLY DISCLAIMS ANY AND ALL WARRANTIES CONCERNING  */
/* THIS SOFTWARE AND DOCUMENTATION, INCLUDING ANY WARRANTIES OF      */
/* MERCHANTABILITY, FITNESS FOR ANY PARTICULAR PURPOSE,              */
/* NON-INFRINGEMENT AND WARRANTIES OF PERFORMANCE, AND ANY WARRANTY  */
/* THAT MIGHT OTHERWISE ARISE FROM COURSE OF DEALING OR USAGE OF     */
/* TRADE. NO WARRANTY IS EITHER EXPRESS OR IMPLIED WITH RESPECT TO   */
/* THE USE OF THE SOFTWARE OR DOCUMENTATION.                         */
/* Under no circumstances shall University be liable for incidental, */
/* special, indirect, direct or consequential damages or loss of     */
/* profits, interruption of business, or related expenses which may  */
/* arise from use of Software or Documentation, including but not    */
/* limited to those resulting from defects in Software and/or        */
/* Documentation, or loss or inaccuracy of data of any kind.         */
/*********************************************************************/

#include <stdio.h>
#include <ctype.h>
#include "common.h"

const static FLOAT dm1 = -1.;

int CNAME(long m, FLOAT *a, long lda, FLOAT *b, long incb, void *buffer){

  long i, is, min_i;
  FLOAT *gemvbuffer = (FLOAT *)buffer;
  FLOAT *B = b;

  if (incb != 1) {
    B = buffer;
    gemvbuffer = (FLOAT *)(((long)buffer + m * sizeof(FLOAT) + 4095) & ~4095);
    COPY_K(m, b, incb, buffer, 1);
  }
  
  for (is = m; is > 0; is -= TRSV_P1){

    min_i = MIN(is, TRSV_P1);

#ifdef TRANSA
    if (m - is > 0){
      GEMV_T(m - is, min_i, 0, dm1, 
	     a + is + (is - min_i) * lda, lda,
	     B + is, 1,
	     B + is - min_i, 1, gemvbuffer);
    }
#endif
    
    for (i = 0; i < min_i; i++) {
      FLOAT *AA = a + (is - i - 1) + (is - i - 1) * lda;
      FLOAT *BB = B + (is - i - 1);

#ifdef TRANSA
      if (i > 0) BB[0] -= DOT_K(i, AA + 1, 1, BB + 1, 1);
#endif

#ifndef UNIT
      BB[0] /= AA[0];
#endif

#ifndef TRANSA
      if (i < min_i - 1) AXPY_K(min_i - i - 1, 0, 0, -BB[0], AA - (min_i - i - 1), 1, BB - (min_i - i - 1), 1, NULL, 0);
#endif
    }

#ifndef TRANSA
    if (is - min_i > 0){
      GEMV_N(is - min_i, min_i, 0, dm1, 
	     a +  (is - min_i) * lda, lda,
	     B + is - min_i, 1,
	     B,              1, gemvbuffer);
    }
#endif
    
  }

  if (incb != 1) {
    COPY_K(m, buffer, 1, b, incb);
  }

  return 0;
}
